/****************************************************************************************/
/*											*/
/* This program is free software; you can redistribute it and/or modify it under the	*/
/* terms of the GNU General Public License as published by the Free Software		*/
/* Foundation; either version 2 of the License, or (at your option) any later version.	*/
/*											*/
/* This program is distributed in the hope that it will be useful, but WITHOUT ANY	*/
/* WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A	*/
/* PARTICULAR PURPOSE. See the GNU General Public License for more details.		*/
/*											*/
/* You should have received a copy of the GNU General Public License along with this	*/
/* program; (See "COPYING"). If not, If not, see <http://www.gnu.org/licenses/>.        */
/*											*/
/*--------------------------------------------------------------------------------------*/
/*											*/
/*  Copyright   Joerg Anders, TU Chemnitz, Fakultaet fuer Informatik, GERMANY           */
/*		ja@informatik.tu-chemnitz.de						*/
/*											*/
/*											*/
/****************************************************************************************/

#ifndef IDIOTSEDITOR_H

#define IDIOTSEDITOR_H
#include "config.h"
#include <sys/time.h>
#include <gtk/gtk.h>
#include <cairo.h>
#include <cairo-features.h>
#include "resource.h"
#include "paths.h"

class NedIdNote {
	public:
		NedIdNote(int f_pitch, unsigned long long st) {
			m_midi_start = st;
			m_valid = false;
			m_pitch = f_pitch;
		}

		void closeNote(unsigned long long  st) {
			if (st > m_midi_start) {
				m_midi_stop = st;
				m_valid = true;
			}
		}

		unsigned long long m_midi_start, m_midi_stop;
		int m_pitch;
		bool m_valid;
};


class NedIdiotsEditor {
	public:
		NedIdiotsEditor(bool with_triplets, bool with_single_triplets, bool with_alto_clef, int voice_num,
			int tempo_change_density, int volume_change_density, bool force_piano, bool force_one_staff);
		~NedIdiotsEditor();
		void closeNote(int pitch, unsigned long long end_time, int midi_nr);
		void addNote(int pitch, unsigned long long  start, int midi_nr);
	private:
		static const char *guiDescription;
		static const GtkActionEntry file_entries[];
		static gboolean handle_expose(GtkWidget *widget, GdkEventExpose *event, gpointer data);
		static gboolean draw_piano(GtkWidget *widget, GdkEventExpose *event, gpointer data);
		static void zoom_in(GtkWidget  *widget, void *data);
		static void zoom_out(GtkWidget  *widget, void *data);
		static void handle_scroll(GtkAdjustment *adjustment, gpointer data);
		static const char *pitch_tab[];
		void draw();
		void repaint();
		double m_zoom_value;
		GtkUIManager *m_ui_manager;
		GtkActionGroup *m_menu_action_group;
		GtkWidget *m_main_window;
		GtkWidget *m_drawing_area;
		GtkWidget *m_piano_area;
		GtkWidget *m_scrolled_window;
		GtkAdjustment *m_hadjust;
		GtkAdjustment *m_vadjust;

		GList *m_notes;

		static void close_i_edit(GtkWidget  *widget, void *data);
};

#endif /* IDIOTSEDITOR_H */
